@@ -23,6 +23,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include <string.h>
 
 #include <gtk/gtkaccellabel.h>
 #include <gtk/gtkimage.h>
@@ -34,6 +35,7 @@
 #include <libxfcegui4/icons.h>
 #include <libxfcegui4/xfce-appmenuitem.h>
 #include <libxfcegui4/xfce-internals.h>
+#include <libxfcegui4/xfce-exec.h>
 
 static void xfce_app_menu_item_class_init(XfceAppMenuItemClass *klass);
 static void xfce_app_menu_item_init(XfceAppMenuItem *app_menu_item);
@@ -48,8 +50,6 @@ static void xfce_app_menu_item_get_property(GObject *object,
 static void xfce_app_menu_item_destroy(GtkObject *object);
 static void xfce_app_menu_item_finalize(GObject *object);
 
-static gint _calc_icon_size();
-static void _ensure_icon_theme_instance();
 static void _command_activate_cb(GtkMenuItem *menu_item,
                                  gpointer user_data);
 
@@ -73,6 +73,20 @@ struct _XfceAppMenuItemPriv {
 static GtkImageMenuItemClass *parent_class = NULL;
 static guint _xfce_app_menu_item_icon_size = 24;
 
+static void
+_style_set_cb(GtkWidget *w, GtkStyle *prev_style, gpointer user_data)
+{
+	GtkStyle *style;
+	GList *children, *l;
+	
+	style = gtk_rc_get_style_by_paths(gtk_settings_get_default(),
+			"GtkMenuItem", "GtkMenuItem", GTK_TYPE_IMAGE_MENU_ITEM);
+	children = gtk_container_get_children(GTK_CONTAINER(w));
+	for(l = children; l; l = l->next)
+		if(GTK_IS_WIDGET(l->data))
+			gtk_widget_set_style(GTK_WIDGET(l->data), style);	
+}
+
 static void
 _command_activate_cb(GtkMenuItem *menu_item, gpointer user_data)
 {
@@ -231,17 +245,6 @@ xfce_app_menu_item_get_property(GObject *object, guint prop_id, GValue *value,
 	}
 }
 
-static void
-xfce_app_menu_item_toggle_size_request(GtkMenuItem *menu_item, gint *requisition)
-{
-	XfceAppMenuItem *app_menu_item = XFCE_APP_MENU_ITEM(menu_item);
-	
-	if(app_menu_item->image_menu_item.image)
-		*requisition = app_menu_item->image_menu_item.image->requisition.width;
-	else
-		*requisition = 0;
-}
-
 static void
 xfce_app_menu_item_destroy(GtkObject *object)
 {
@@ -294,6 +297,8 @@ xfce_app_menu_item_new()
 	
 	g_signal_connect(G_OBJECT(app_menu_item), "activate",
 			G_CALLBACK(_command_activate_cb), NULL);
+	g_signal_connect(G_OBJECT(app_menu_item), "style-set",
+			G_CALLBACK(_style_set_cb), NULL);
 	
 	return GTK_WIDGET(app_menu_item);
 }
@@ -313,21 +318,18 @@ xfce_app_menu_item_new_with_label(const gchar *label)
 	XfceAppMenuItem *app_menu_item;
 	GtkWidget *accel_label;
 	
-	app_menu_item = g_object_new(XFCE_TYPE_APP_MENU_ITEM, NULL);
+	app_menu_item = XFCE_APP_MENU_ITEM(xfce_app_menu_item_new());
 	
 	app_menu_item->priv->name = g_strdup(label);
 	
 	accel_label = gtk_accel_label_new(app_menu_item->priv->name);
 	gtk_misc_set_alignment(GTK_MISC(accel_label), 0.0, 0.5);
-
+	
 	gtk_container_add(GTK_CONTAINER(app_menu_item), accel_label);
 	gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(accel_label),
 			GTK_WIDGET(app_menu_item));
 	gtk_widget_show(accel_label);
 	
-	g_signal_connect(G_OBJECT(app_menu_item), "activate",
-			G_CALLBACK(_command_activate_cb), NULL);
-
 	return GTK_WIDGET(app_menu_item);
 }
 
@@ -349,7 +351,7 @@ xfce_app_menu_item_new_with_mnemonic(const gchar *label)
 	XfceAppMenuItem *app_menu_item;
 	GtkWidget *accel_label;
 	
-	app_menu_item = g_object_new(XFCE_TYPE_APP_MENU_ITEM, NULL);
+	app_menu_item = XFCE_APP_MENU_ITEM(xfce_app_menu_item_new());
 	
 	app_menu_item->priv->name = g_strdup(label);
 	
@@ -357,15 +359,12 @@ xfce_app_menu_item_new_with_mnemonic(const gchar *label)
 	gtk_label_set_text_with_mnemonic(GTK_LABEL(accel_label),
 			app_menu_item->priv->name);
 	gtk_misc_set_alignment(GTK_MISC(accel_label), 0.0, 0.5);
-
+	
 	gtk_container_add(GTK_CONTAINER(app_menu_item), accel_label);
 	gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(accel_label),
 			GTK_WIDGET(app_menu_item));
 	gtk_widget_show(accel_label);
 
-	g_signal_connect(G_OBJECT(app_menu_item), "activate",
-			G_CALLBACK(_command_activate_cb), NULL);
-	
 	return GTK_WIDGET(app_menu_item);
 }
 
@@ -386,21 +385,19 @@ xfce_app_menu_item_new_with_command(const gchar *label, const gchar *command)
 	XfceAppMenuItem *app_menu_item;
 	GtkWidget *accel_label;
 	
-	app_menu_item = g_object_new(XFCE_TYPE_APP_MENU_ITEM, NULL);
+	app_menu_item = XFCE_APP_MENU_ITEM(xfce_app_menu_item_new());
 	
 	app_menu_item->priv->name = g_strdup(label);
 	
 	accel_label = gtk_accel_label_new(app_menu_item->priv->name);
 	gtk_misc_set_alignment(GTK_MISC(accel_label), 0.0, 0.5);
-
+	
 	gtk_container_add(GTK_CONTAINER(app_menu_item), accel_label);
 	gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(accel_label),
 			GTK_WIDGET(app_menu_item));
 	gtk_widget_show(accel_label);
 	
 	app_menu_item->priv->command = xfce_expand_variables(command, NULL);
-	g_signal_connect(G_OBJECT(app_menu_item), "activate",
-			G_CALLBACK(_command_activate_cb), NULL);
 	
 	return GTK_WIDGET(app_menu_item);
 }
@@ -468,7 +465,7 @@ xfce_app_menu_item_new_from_desktop_entry(XfceDesktopEntry *entry,
 	
 	g_return_val_if_fail(XFCE_IS_DESKTOP_ENTRY(entry), NULL);
 	
-	app_menu_item = g_object_new(XFCE_TYPE_APP_MENU_ITEM, NULL);
+	app_menu_item = XFCE_APP_MENU_ITEM(xfce_app_menu_item_new());
 	
 	if(!xfce_desktop_entry_get_string(entry, "Name", TRUE, &name)) {
 		gchar *tmp, *tmp1;
@@ -486,7 +483,7 @@ xfce_app_menu_item_new_from_desktop_entry(XfceDesktopEntry *entry,
 	
 	accel_label = gtk_accel_label_new(app_menu_item->priv->name);
 	gtk_misc_set_alignment(GTK_MISC(accel_label), 0.0, 0.5);
-
+	
 	gtk_container_add(GTK_CONTAINER(app_menu_item), accel_label);
 	gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(accel_label),
 			GTK_WIDGET(app_menu_item));
@@ -519,9 +516,6 @@ xfce_app_menu_item_new_from_desktop_entry(XfceDesktopEntry *entry,
 		}
 	}
 	
-	g_signal_connect(G_OBJECT(app_menu_item), "activate",
-			G_CALLBACK(_command_activate_cb), NULL);
-	
 	return GTK_WIDGET(app_menu_item);
 }
 
@@ -564,7 +558,6 @@ xfce_app_menu_item_set_icon_name(XfceAppMenuItem *app_menu_item,
 {
 	GtkWidget *image;
 	GdkPixbuf *pix;
-	gchar *dot;
 	gint w, h;
 	
 	g_return_if_fail(XFCE_IS_APP_MENU_ITEM(app_menu_item));
